from pytest import raises
from .messages import Message
from .ports import BaseIOPort

def test_ioport():
    class Port(BaseIOPort):
        def _open(self):
            self.close_called = False

        def _send(self, message):
            self._messages.append(message)

        def _close(self):
            self.close_called = True

    with Port('Name') as port:
        assert port.name == 'Name'
        assert not port.closed

        assert port._messages is port._parser.messages

        with raises(TypeError): port.send('not a message')

    with Port('Name') as port:
        message = Message('note_on')

        # Send and receive a message. (Blocking.)
        #port.send(message)
        #message2 = port.receive()
        #raise ValueError(('ost', message, message2))

        # Receive a message. (Non-blocking.)
        port.send(message)
        message2 = port.poll()
        assert port.poll() is None

    with Port('Name') as port:
        message = Message('note_on')

        port.send(message)
        port.send(message)
        assert port.pending() == 2
        assert list(port.iter_pending()) == [message, message]

    with Port('Name') as port:
        port.close()
        assert port.close_called
        port.close_called = False
        port.close()
        assert port.closed
        assert not port.close_called

        # Todo: should this type of port close (and/or stop iteration)
        # when there are no messages?
        # port.send(message)
        # port.send(message)
        # assert port.pending() == 2
        # assert list(port) == [message, message]

def test_close_inside_iteration():
    # This type of port can close when it runs out of messages.
    # (And example of this is socket ports.)
    #
    # Iteration should then stop after all messages in the
    # internal queue have been received.
    message = Message('note_on')

    class Port(BaseIOPort):
        def __init__(self, messages):
            BaseIOPort.__init__(self)
            # Simulate some messages that arrived
            # earlier.
            self._messages.extend(messages)
            self.closed = False

        def _receive(self, block=True):
            # Oops, the other end hung up.
            if self._messages:
                return self._messages.popleft()
            else:
                self.close()
                return None

    message = Message('note_on')
    with Port([message, message]) as port:
        assert len(list(port)) == 2



import sys
from pytest import raises
from .messages import Message, parse_time
from .messages import parse_string, format_as_string, parse_string_stream

PY2 = (sys.version_info.major == 2)

if PY2:
    from StringIO import StringIO
else:
    from io import StringIO

def test_parse_string():
    assert (parse_string('note_on channel=2 note=3')
            == Message('note_on', channel=2, note=3))

    assert (parse_string('sysex data=(1,2,3)')
            == Message('sysex', data=(1, 2, 3)))

    assert (parse_string('note_on time=0.5').time
            == Message('note_on', time=0.5).time)

    # nan and inf should be allowed
    assert parse_string('note_on time=inf').time == float('inf')
    assert str(parse_string('note_on time=nan').time) == 'nan'

    # Commas are not allowed
    with raises(ValueError): parse_string('note_on channel=2, note=3')

    with raises(ValueError): parse_string('++++S+S+SOIO(KOPKEPOKFWKF')
    with raises(ValueError): parse_string('note_on banana=2')
    with raises(ValueError): parse_string('sysex (1, 2, 3)')
    with raises(ValueError): parse_string('sysex (1  2  3)')

def test_format_as_string():
    assert (format_as_string(Message('note_on', channel=9))
            == 'note_on channel=9 note=0 velocity=64 time=0')

    assert (format_as_string(Message('sysex', data=(1, 2, 3)))
            == 'sysex data=(1,2,3) time=0')

    assert (format_as_string(Message('sysex', data=()))
            == 'sysex data=() time=0')

    assert (format_as_string(Message('continue'))
            == 'continue time=0')

def test_parse_string_stream():
    # Correct input.
    stream = StringIO("""
         note_on channel=1  # Ignore this
         # and this
         continue
    """)
    gen = parse_string_stream(stream)
    assert next(gen) == (Message('note_on', channel=1), None)
    assert next(gen) == (Message('continue'), None)

    # Invalid input. It should catch the ValueError
    # from parse_string() and return (None, 'Error message').
    stream = StringIO('ijsoijfdsf\noiajoijfs')
    gen = parse_string_stream(stream)
    assert next(gen)[0] == None
    assert next(gen)[0] == None
    with raises(StopIteration): next(gen)

def test_parse_string_time():
    # These should work:
    parse_time('0')
    parse_time('12')
    parse_time('-9')
    parse_time('0.5')
    parse_time('10e10')
    parse_time('inf')
    parse_time('-inf')
    parse_time('nan')
    parse_time('2389284878375')  # Will be a long in Python 2

    # These should not
    with raises(ValueError): parse_time('banana')
    with raises(ValueError): parse_time('None')
    with raises(ValueError): parse_time('-')
    with raises(ValueError): parse_time('938938958398593L')



#!/usr/bin/env python
"""
Example of non-blocking reception from input port.
"""

from __future__ import print_function
import sys
import time
import mido

if len(sys.argv) > 1:
    portname = sys.argv[1]
else:
    portname = None  # Use default port

try:
    with mido.open_input(portname) as port:
        print('Using {}'.format(port))        
        while True:
            for message in port.iter_pending():
                print('Received {}'.format(message))

            print('Doing something else for a while...')
            time.sleep(0.5)
except KeyboardInterrupt:
    pass



"""
MIDI messages

There is no need to use this module directly. All you need is
available in the top level module.
"""
import sys
from numbers import Real, Integral

PY2 = (sys.version_info.major == 2)

# Pitchwheel is a 14 bit signed integer
MIN_PITCHWHEEL = -8192
MAX_PITCHWHEEL = 8191

# Song pos is a 14 bit unsigned integer
MIN_SONGPOS = 0
MAX_SONGPOS = 16383

class MessageSpec(object):
    """
    Specifications for creating a message.
    
    status_byte is the first byte of the message. For channel
    messages, the channel (lower 4 bits) is clear.

    type is the type name of the message, for example 'sysex'.

    arguments is the attributes / keywords arguments specific to
    this message type.

    length is the length of this message in bytes. This value is not used
    for sysex messages, since they use an end byte instead.

    Table of MIDI messages:

        http://www.midi.org/techspecs/midimessages.php
    """

    def __init__(self, status_byte, type_, arguments, length):
        """Create a new message specification."""
        self.status_byte = status_byte
        self.type = type_
        self.arguments = arguments
        self.length = length
   
        # Attributes that can be set on the object
        self.settable_attributes = set(self.arguments) | {'time'}

    def signature(self):
        """Return call signature for Message constructor for this type.

        The signature is returned as a string.
        """
        parts = []
        parts.append(repr(self.type))

        for name in self.arguments:
            if name == 'data':
                parts.append('data=()')
            else:
                parts.append('{}=0'.format(name))
        parts.append('time=0')

        sig = '({})'.format(', '.join(parts))

        return sig


def get_message_specs():
    return [
        # Channel messages
        MessageSpec(0x80, 'note_off', ('channel', 'note', 'velocity'), 3),
        MessageSpec(0x90, 'note_on', ('channel', 'note', 'velocity'), 3),
        MessageSpec(0xa0, 'polytouch', ('channel', 'note', 'value'), 3),
        MessageSpec(0xb0, 'control_change',
                    ('channel', 'control', 'value'), 3),
        MessageSpec(0xc0, 'program_change', ('channel', 'program',), 2),
        MessageSpec(0xd0, 'aftertouch', ('channel', 'value',), 2),
        MessageSpec(0xe0, 'pitchwheel', ('channel', 'pitch',), 3),

        # System common messages
        MessageSpec(0xf0, 'sysex', ('data',), float('inf')),
        MessageSpec(0xf1, 'quarter_frame', ('frame_type', 'frame_value'), 2),
        MessageSpec(0xf2, 'songpos', ('pos',), 3),
        MessageSpec(0xf3, 'song_select', ('song',), 2),
        # 0xf4 is undefined.
        # 0xf5 is undefined.
        MessageSpec(0xf6, 'tune_request', (), 1),
        # 0xf7 is the stop byte for sysex messages, so should not be a message.

        # System real time messages
        MessageSpec(0xf8, 'clock', (), 1),
        # 0xf9 is undefined.
        MessageSpec(0xfa, 'start', (), 1),
        MessageSpec(0xfb, 'continue', (), 1),
        MessageSpec(0xfc, 'stop', (), 1),
        # 0xfd is undefined.
        MessageSpec(0xfe, 'active_sensing', (), 1),
        MessageSpec(0xff, 'reset', (), 1),
    ]


def build_spec_lookup(message_specs):
    lookup = {}

    for spec in message_specs:
        status_byte = spec.status_byte

        if status_byte < 0xf0:
            # Channel message.
            # The upper 4 bits are message type, and
            # the lower 4 are MIDI channel.
            # We need lookup for all 16 MIDI channels.
            for channel in range(16):
                lookup[status_byte | channel] = spec
        else:
            lookup[status_byte] = spec

        lookup[spec.type] = spec

    return lookup


def get_spec(type_or_status_byte):
    """Get message specification from status byte or message type name.

    For use in writing parsers.
    """
    try:
        return Message._spec_lookup[type_or_status_byte]
    except KeyError:
        raise LookupError('unknown type or status byte')


def check_time(time):
    """Check type and value of time.
    
    Raises TypeError if value is not an integer or a float
    """
    if not isinstance(time, Real):
        raise TypeError('time must be an integer or float')


def check_channel(channel):
    """Check type and value of channel.

    Raises TypeError if the value is not an integer, and ValueError if
    it is outside range 0..15.
    """
    if not isinstance(channel, Integral):
        raise TypeError('channel must be an integer')
    elif not 0 <= channel <= 15:
        raise ValueError('channel must be in range 0..15')


def check_pos(pos):
    """Check type and value of song position.

    Raise TypeError if the value is not an integer, and ValueError if
    it is outside range MIN_SONGPOS..MAX_SONGPOS.
    """
    if not isinstance(pos, Integral):
        raise TypeError('song pos must be and integer')
    elif not MIN_SONGPOS <= pos <= MAX_SONGPOS:
        raise ValueError('song pos must be in range {}..{}'.format(
                         MIN_SONGPOS, MAX_SONGPOS))


def check_pitch(pitch):
    """Raise TypeError if the value is not an integer, and ValueError
    if it is outside range MIN_PITCHWHEEL..MAX_PITCHWHEEL.
    """
    if not isinstance(pitch, Integral):
        raise TypeError('pichwheel value must be an integer')
    elif not MIN_PITCHWHEEL <= pitch <= MAX_PITCHWHEEL:
        raise ValueError('pitchwheel value must be in range {}..{}'.format(
                         MIN_PITCHWHEEL, MAX_PITCHWHEEL))


def check_data(data_bytes):
    """Check type of data_byte and type and range of each data byte.

    Returns the data bytes as a SysexData object.

    Raises TypeError if value is not iterable.
    Raises TypeError if one of the bytes is not an integer.
    Raises ValueError if one of the bytes is out of range 0..127.
    """
    # Make the sequence immutable.
    data_bytes = SysexData(data_bytes)

    for byte in data_bytes:
        check_databyte(byte)

    return data_bytes


def check_frame_type(value):
    """Check type and value SMPTE quarter frame type.

    Raises TypeError if the value is not an integer.
    Raises ValueError if the value is out of range.
    """
    if not isinstance(value, Integral):
        raise TypeError('frame_type must be an integer')
    elif not 0 <= value <= 7:
        raise ValueError('frame_type must be in range 0..7')


def check_frame_value(value):
    """Check type and value of SMPTE quarter frame value.

    Raises TypeError if the value is not an integer.
    Raises ValueError if the value is out of range.
    """
    if not isinstance(value, Integral):
        raise TypeError('frame_value must be an integer')
    elif not 0 <= value <= 15:
        raise ValueError('frame_value must be in range 0..15')


def check_databyte(value):
    """Raise exception of byte has wrong type or is out of range

    Raises TypeError if the byte is not an integer, and ValueError if
    it is out of range. Data bytes are 7 bit, so the valid range is
    0..127.
    """
    if not isinstance(value, Integral):
        raise TypeError('data byte must be an integer')
    elif not 0 <= value <= 127:
        raise ValueError('data byte must be in range 0..127')


def encode_channel(channel):
    """Convert channel into a list of bytes. Return an empty list of
    bytes, since channel is already masked into status byte.
    """
    return []


def encode_data(data):
    """Encode sysex data as a list of bytes. A sysex end byte (0xf7)
    is appended.
    """
    return list(data) + [0xf7]

 
def encode_pitch(pitch):
    """Encode pitchwheel pitch as a list of bytes."""
    pitch -= MIN_PITCHWHEEL
    return [pitch & 0x7f, pitch >> 7]


def encode_pos(pos):
    """Encode song position as a list of bytes."""
    return [pos & 0x7f, pos >> 7]


class BaseMessage(object):
    """Base class for MIDI messages.

    Can be subclassed to create meta messages, for example.
    """
    def copy(self, **overrides):
        """Return a copy of the message.

        Attributes will be overridden by the passed keyword arguments.
        Only message specific attributes can be overridden. The message
        type can not be changed.

        Example:

            a = Message('note_on')
            b = a.copy(velocity=32)
        """

        # Make an exact copy of this object.
        klass = self.__class__
        msg = klass.__new__(klass)
        vars(msg).update(vars(self))

        for name, value in overrides.items():
            try:
                # setattr() is responsible for checking the
                # name and type of the attribute.
                setattr(msg, name, value)
            except AttributeError as err:
                raise ValueError(*err.args)

        return msg

    def bytes(self):
        raise ValueError('bytes() is not implemented in this class')

    def bin(self):
        """Encode message and return as a bytearray.

        This can be used to write the message to a file.
        """
        return bytearray(self.bytes())

    def hex(self, sep=' '):
        """Encode message and return as a string of hex numbers,

        Each number is separated by the string sep.
        """
        return sep.join('{:02X}'.format(byte) for byte in self.bytes())

    def __eq__(self, other):
        """Compare message to another for equality."""
        if not isinstance(other, BaseMessage):
            raise TypeError('comparison between message and another type')

        return vars(self) == vars(other)


class SysexData(tuple):
    """Special kind of tuple accepts and converts any sequence in +=."""
    def __iadd__(self, other):
        return SysexData(self + check_data(other))


class Message(BaseMessage):
    """
    MIDI message class.
    """

    # Quick lookup of specs by name or status_byte.
    _spec_lookup = build_spec_lookup(get_message_specs())

    # This is needed for __init__() so it doesn't accept status bytes.
    _type_lookup = {name: value for name, value in _spec_lookup.items() \
                        if not isinstance(name, int)}

    def __init__(self, type, **arguments):
        """Create a new message.

        The first argument is typically the type of message to create,
        for example 'note_on'.
        """
        try:
            spec = self._type_lookup[type]
        except KeyError:
            raise ValueError('invalid message type {!r}'.format(type))

        vars(self)['type'] = type
        vars(self)['_spec'] = spec

        # Set default values.
        for name in spec.arguments:
            if name == 'velocity':
                vars(self)['velocity'] = 0x40
            elif name == 'data':
                vars(self)['data'] = SysexData()
            else:
                vars(self)[name] = 0
        vars(self)['time'] = 0

        # Override defaults.
        for name, value in arguments.items():
            try:
                setattr(self, name, value)
            except AttributeError as err:
                raise ValueError(*err.args)

    def __setattr__(self, name, value):
        if name in self._spec.settable_attributes:
            try:
                if name == 'data':
                    value = check_data(value)
                else:
                    globals()['check_{}'.format(name)](value)
            except KeyError:
                check_databyte(value)

            vars(self)[name] = value
        elif name in vars(self):
            raise AttributeError('{} attribute is read only'.format(name))
        else:
            raise AttributeError(
                '{} message has no attribute {}'.format(self.type, name))

    def __delattr__(self, name):
        raise AttributeError('attribute cannot be deleted')

    def bytes(self):
        """Encode message and return as a list of integers."""

        status_byte = self._spec.status_byte
        if status_byte < 0xf0:
            # Add channel (lower 4 bits) to status byte.
            # Those bits in spec.status_byte are always 0.
            status_byte |= self.channel

        message_bytes = [status_byte]

        if self.type == 'quarter_frame':
            message_bytes.append(self.frame_type << 4 | self.frame_value)
        else:
            for name in self._spec.arguments:
                value = getattr(self, name)
                try:
                    encode = globals()['encode_{}'.format(name)]
                    message_bytes.extend(encode(value))
                except KeyError:
                    message_bytes.append(value)

        return message_bytes

    def __repr__(self):
        parts = []

        for name in self._spec.arguments + ('time',):
            parts.append('{}={!r}'.format(name, getattr(self, name)))

        return '<message {} {}>'.format(self.type, ' '.join(parts))

    def __str__(self):
        return format_as_string(self)

    def __len__(self):
        if self.type == 'sysex':
            return len(self.data) + 2
        else:
            return self._spec.length


def build_message(spec, bytes, time=0):
    """Build message from bytes.

    This is used by Parser and MidiFile. bytes is a full list
    of bytes for the message including the status byte. For sysex
    messages, the end byte is not included. Examples:

        build_message(spec, [0x80, 20, 100])
        build_message(spec, [0xf0, 1, 2, 3])

    No type or value checking is done, so you need to do that before
    you call this function. (This includes time.) 0xf7 is not allowed
    as status byte.
    """
    # This could be written in a more general way, but most messages
    # are note_on or note_off so doing it this way is faster.
    if spec.type in ['note_on', 'note_off']:
        attributes = {
                'channel': bytes[0] & 0x0f,
                'note': bytes[1],
                'velocity': bytes[2],
                }

    elif spec.type == 'control_change':
        attributes = {
                'channel': bytes[0] & 0x0f,
                'control': bytes[1],
                'value': bytes[2],
                }

    elif spec.status_byte < 0xf0:
        # Channel message. The most common type.
        if spec.type == 'pitchwheel':
            pitch = bytes[1] | ((bytes[2] << 7) + MIN_PITCHWHEEL)
            attributes = {'pitch': pitch}
        else:
            attributes = dict(zip(spec.arguments, bytes))
        # Replace status_bytes sneakily with channel.
        attributes['channel'] = bytes[0] & 0x0f

    elif spec.type == 'sysex':
        attributes = {'data': tuple(bytes[1:])}

    elif spec.type == 'songpos':
        pos = bytes[1] | (bytes[2] << 7)
        attributes = {'pos': pos}

    elif spec.type == 'quarter_frame':
        attributes = {'frame_type': bytes[1] >> 4,
                     'frame_value' : bytes[1] & 15}

    else:
        attributes = dict(zip(spec.arguments, bytes[1:]))

    # Message.__new__() is used as an optimization to
    # get around argument checking. We already know that
    # the values are valid.
    msg = Message.__new__(Message)
    vars(msg).update(attributes)
    vars(msg).update({
        'type': spec.type,
        '_spec': spec,
        'time': time,
    })
    return msg

def parse_time(text):
    if text.endswith('L'):
        raise ValueError('L is not allowed in time')

    if PY2:
        converters = [int, long, float]
    else:
        converters = [int, float]

    for convert in converters:
        try:
            return convert(text)
        except ValueError:
            pass

    raise ValueError('invalid format for time')


def parse_string(text):
    """Parse a string of text and return a message.

    The string can span multiple lines, but must contain
    one full message.

    Raises ValueError if the string could not be parsed.
    """
    words = text.split()

    type_name = words[0]
    arguments = words[1:]

    names_seen = set()

    kwargs = {}

    for argument in arguments:
        try:
            name, value = argument.split('=')
        except ValueError:
            raise ValueError('missing or extraneous equals sign')

        if name in names_seen:
            raise ValueError('argument passed more than once')
        names_seen.add(name)

        if name == 'data':
            if not value.startswith('(') and value.endswith(')'):
                raise ValueError('missing parentheses in data message')

            try:
                data_bytes = [int(byte) for byte in value[1:-1].split(',')]
            except ValueError:
                raise ValueError('unable to parse data bytes')
            kwargs['data'] = data_bytes
        elif name == 'time':
            try:
                time = parse_time(value)
            except ValueError:
                raise ValueError('invalid value for time')
            try:
                kwargs['time'] = time
            except AttributeError as err:
                raise ValueError(err.message)
            except TypeError as err:
                raise ValueError(err.message)
        else:
            try:
                kwargs[name] = int(value)
            except AttributeError as exception:
                raise ValueError(*exception.args)
            except ValueError:
                raise ValueError('{!r} is not an integer'.format(value))

    return Message(type_name, **kwargs)


def parse_string_stream(stream):
    """Parse a stram of messages and yield (message, error_message)

    stream can be any iterable that generates text strings, where each
    string is a string encoded message.

    If a string can be parsed, (message, None) is returned. If it
    can't be parsed (None, error_message) is returned. The error
    message containes the line number where the error occurred.
    """
    line_number = 1
    for line in stream:
        try:
            line = line.split('#')[0].strip()
            if line:
                yield parse_string(line), None
        except ValueError as exception:
            error_message = 'line {line_number}: {message}'.format(
                line_number=line_number,
                message=exception.args[0])
            yield None, error_message
        line_number += 1


def format_as_string(message, include_time=True):
    """Format a message and return as a string.

    This is equivalent to str(message).

    To leave out the time attribute, pass include_time=False.
    """
    if not isinstance(message, Message):
        raise ValueError('message must be a mido.Message object')

    words = []
    words.append(message.type)

    names = message._spec.arguments
    if include_time:
        names += ('time',)

    for name in names:
        value = getattr(message, name)
        if name == 'data':
            value = '({})'.format(','.join(str(byte) for byte in value))
        elif name == 'time':
            # Python 2 formats longs as '983989385L'. This is not allowed.
            value = str(value)
            value = value.replace('L', '')
        words.append('{}={}'.format(name, value))
    
    return ' '.join(words)



import sys
from pytest import raises

from .messages import Message, check_time, get_spec
from .messages import MIN_PITCHWHEEL, MAX_PITCHWHEEL
from .parser import Parser, parse, parse_all

PY2 = (sys.version_info.major == 2)

def test_msg_equality():
    args = dict(type='note_on', channel=1, note=2, velocity=3)
    assert Message(**args) == Message(**args)
    assert Message(time=1, **args) != Message(time=2, **args)

def test_set_type():
    with raises(AttributeError):
        Message('note_on').type = 'note_off'

def test_pitchwheel():
    """Check if pitchwheel type check and encoding is working."""
    msg = Message('pitchwheel', pitch=MIN_PITCHWHEEL)
    bytes = msg.bytes()
    assert bytes[1] == bytes[2]

    msg = Message('pitchwheel', pitch=MAX_PITCHWHEEL)
    bytes = msg.bytes()
    assert bytes[1] == bytes[2] == 127

def test_pitchwheel_encode_parse():
    """Encode and parse pitchwheel with value=0."""
    wheel = Message('pitchwheel', pitch=0)
    assert wheel == parse(wheel.bytes())

def test_quarter_frame_encode_parse():
    """Encode and parse quarter_frame."""
    frame = Message('quarter_frame', frame_type=1, frame_value=2)
    assert frame == parse(frame.bytes())

def test_sysex():
    sysex = Message('sysex', data=(1, 2, 3, 4, 5))
    assert sysex == parse(sysex.bytes())

def test_check_time():
    check_time(1)
    check_time(1.5)

    if PY2:
        # long should be allowed. (It doesn't exist in Python3,
        # so there's no need to check for it here.)
        check_time(long('9829389283L'))

    with raises(TypeError): check_time(None)
    with raises(TypeError): check_time('abc')

    with raises(TypeError): check_time(None)
    with raises(TypeError): check_time('abc')

def test_check_channel():
    from .messages import check_channel

    check_channel(0)
    check_channel(15)
    with raises(TypeError): check_channel(None)
    with raises(TypeError): check_channel('abc')
    with raises(TypeError): check_channel(0.5)
    with raises(ValueError): check_channel(-1)
    with raises(ValueError): check_channel(16)

def test_check_pos():
    from .messages import check_pos, MIN_SONGPOS, MAX_SONGPOS

    check_pos(0)
    check_pos(MIN_SONGPOS)
    check_pos(MAX_SONGPOS)
    with raises(TypeError): check_pos(None)
    with raises(TypeError): check_pos('abc')
    with raises(ValueError): check_pos(MIN_SONGPOS - 1)
    with raises(ValueError): check_pos(MAX_SONGPOS + 1)

def test_check_pitch():
    from .messages import check_pitch, MIN_PITCHWHEEL, MAX_PITCHWHEEL

    # Pitchwheel pitch
    check_pitch(MIN_PITCHWHEEL)
    check_pitch(MAX_PITCHWHEEL)
    with raises(TypeError): check_pitch(None)
    with raises(TypeError): check_pitch(0.5)
    with raises(TypeError): check_pitch('abc')
    with raises(ValueError): check_pitch(MIN_PITCHWHEEL - 1)
    with raises(ValueError): check_pitch(MAX_PITCHWHEEL + 1)

def test_check_data():
    from .messages import check_data

    # check_data() should return the data as a tuple.
    assert type(check_data([0, 1, 2]) == tuple)
    assert check_data([0, 1, 2]) == (0, 1, 2)
    assert check_data(i for i in [1, 2, 3]) == (1, 2, 3)

    with raises(TypeError): check_data(1)
    with raises(TypeError): check_data(('a', 'b', 'c'))
    with raises(ValueError): check_data((-1, -2, -3))

def test_check_frame_type():
    from .messages import check_frame_type

    # Qarter frame type
    check_frame_type(0)
    check_frame_type(7)
    with raises(TypeError): check_frame_type(None)
    with raises(TypeError): check_frame_type(0.5)
    with raises(ValueError): check_frame_type(-1)
    with raises(ValueError): check_frame_type(8)

def test_check_databyte():
    from .messages import check_databyte

    # Data byte
    check_databyte(0)
    check_databyte(15)
    with raises(TypeError): check_databyte(None)
    with raises(TypeError): check_databyte(0.5)
    with raises(ValueError): check_databyte(-1)
    with raises(ValueError): check_databyte(128)

def test_encode_channel():
    from .messages import encode_channel

    # Channel should be ignored, and an empty list returned.
    # Thus, there is no reason to check for TypeError
    # and ValueError.
    assert encode_channel(channel=0) == []

def test_encode_data():
    from .messages import encode_data

    # Note: encode_data() includes the sysex end byte (0xf7) to avoid a
    # special case in bytes().
    assert encode_data([1, 2, 3]) == [1, 2, 3, 0xf7]

def test_encode_():
    from .messages import encode_pitch, MIN_PITCHWHEEL, MAX_PITCHWHEEL

    # Pitchwheel pitch
    assert encode_pitch(MIN_PITCHWHEEL) == [0, 0]
    assert encode_pitch(MAX_PITCHWHEEL) == [127, 127]
    assert encode_pitch(0) == [0, 64]

def test_encode_pos():
    from .messages import encode_pos, MIN_SONGPOS, MAX_SONGPOS

    assert encode_pos(MIN_SONGPOS) == [0, 0]
    assert encode_pos(MAX_SONGPOS) == [127, 127]
    # Check endian
    assert [16, 78] == encode_pos(10000)

def test_get_spec():
    from .messages import get_spec

    assert get_spec('note_on').type == 'note_on'
    assert get_spec(0x80).type == 'note_off'
    assert get_spec(0x82).type == 'note_off'

    with raises(LookupError): get_spec(-1)
    with raises(LookupError): get_spec(0)
    with raises(LookupError): get_spec('banana')

def test_sysex_data_type():
    """Is messages.data turned into a tuple?"""
    data = range(1)

    message = Message('sysex')
    message.data = data
    assert isinstance(message.data, tuple)

    message = Message('sysex', data=data)
    assert isinstance(message.data, tuple)

    a = Message('sysex', data=(1, 2))
    b = parse(a.bytes())
    assert isinstance(b.data, tuple)

def test_copy():
    orig = Message('note_on', note=22, time=123)
    copy = orig.copy()

    assert orig == copy
    assert orig.time == copy.time
    assert vars(orig) == vars(copy)

    copy = orig.copy(velocity=1)
    orig.velocity = 1

    assert orig == copy

def test_copy_invalid_attribute():
    orig = Message('note_on')
    valid_spec = get_spec('note_on')

    # Pass arguments with invalid names.
    with raises(ValueError): orig.copy(_spec=valid_spec)
    with raises(ValueError): orig.copy(type='continue')
    with raises(ValueError): orig.copy(banana=1)

    # Valid arguments should pass.
    orig.copy(note=0, velocity=0, time=0)

def test_set_invalid_attribute():
    """Set an attribute that is not settable."""
    valid_spec = get_spec('note_on')
    msg = Message('note_on')

    with raises(AttributeError): msg._spec = valid_spec
    with raises(AttributeError): msg.type = 'continue'
    with raises(AttributeError): msg.invalid = 'banana'



"""Backend for rtmidi-python:

https://pypi.python.org/pypi/rtmidi-python

To use this backend copy (or link) it to somewhere in your Python path
and call:

    mido.set_backend('rtmidi_python_backend')

or set shell variable $MIDO_BACKEND to rtmidi_python_backend:

Todo:

* add support for APIs.

* active_sensing is still filtered. (The same is true for
  mido.backends.rtmidi.)There may be a way to remove this filtering.

"""
from __future__ import absolute_import
import os
import time
from Queue import Queue
import rtmidi_python as rtmidi
# Todo: change this to a relative import if the backend is included in
# the package.
from mido.ports import BaseInput, BaseOutput

def get_devices(api=None, **kwargs):
    devices = []

    input_names = set(rtmidi.MidiIn().ports)
    output_names = set(rtmidi.MidiOut().ports)

    for name in sorted(input_names | output_names):
        devices.append({
                'name': name,
                'is_input': name in input_names,
                'is_output': name in output_names,
                })

    return devices

class PortCommon(object):
    def _open(self, api=None, virtual=False, **kwargs):

        self._queue = Queue()

        # rtapi = _get_api_id(api)
        opening_input = hasattr(self, 'receive')
        
        if opening_input:
            self._rt = rtmidi.MidiIn(api)
            self._rt.ignore_types(False, False, True)
            self.callback = kwargs.get('callback')
        else:
            self._rt = rtmidi.MidiOut()  # rtapi=rtapi)
            # Turn of ignore of sysex, time and active_sensing.

        ports = self._rt.ports

        if virtual:
            if self.name is None:
                raise IOError('virtual port must have a name')
            self._rt.open_virtual_port(self.name)
        else:
            if self.name is None:
                # Todo: this could fail if list is empty.
                # In RtMidi, the default port is the first port.
                try:
                    self.name = ports[0]
                except IndexError:
                    raise IOError('no ports available')

            try:
                port_id = ports.index(self.name)
            except ValueError:
                raise IOError('unknown port {!r}'.format(self.name))

            try:
                self._rt.open_port(port_id)
            except RuntimeError as err:
                raise IOError(*err.args)

        # api = _api_to_name[self._rt.get_current_api()]
        api = ''
        self._device_type = 'RtMidi/{}'.format(api)
        if virtual:
            self._device_type = 'virtual {}'.format(self._device_type)

    @property
    def callback(self):
        return self._rt._callback

    @callback.setter
    def callback(self, func):
        self._rt.callback = func

    def _callback_wrapper(self, *args, **kw):
        asodosaid

        self._parser.feed(message_data[0])
        for message in self._parser:
            if self.callback:
                self.callback(message)

    def _close(self):
        self._rt.close_port()
        del self._rt  # Virtual ports are closed when this is deleted.

class Input(PortCommon, BaseInput):
    def _receive(self, block=True):
        # Since there is no blocking read in RtMidi, the block
        # flag is ignored and the enclosing receive() takes care
        # of blocking.

        while True:
            message_data, timestamp = self._rt.get_message()
            if message_data is None:
                break
            else:
                self._parser.feed(message_data)

class Output(PortCommon, BaseOutput):
    def _send(self, message):
        self._rt.send_message(message.bytes())



#!/usr/bin/env python
"""                                                                            
Play MIDI file on output port.

Run with (for example):

    ./play_midi_file.py 'SH-201 MIDI 1' 'test.mid'
"""

import sys
import mido
from mido import MidiFile

filename = sys.argv[1]
if len(sys.argv) == 3:
    portname = sys.argv[2]
else:
    portname = None

with mido.open_output(portname) as output:
    try:
        for message in MidiFile(filename).play():
            print(message)
            output.send(message)

    except KeyboardInterrupt:
        print()
        output.reset()



#!/usr/bin/env python
"""
Receive messages on the input port and print messages with a specific
channel (defaults to 0).

Usage:

    python example_input_filter.py portname [CHANNEL] [...]
"""

from __future__ import print_function
import sys
import mido

def accept_notes(port):
    """Only let note_on and note_off messages through."""
    for message in port:
        if message.type in ('note_on', 'note_off'):
            yield message


if len(sys.argv) > 1:
    portname = sys.argv[1]
else:
    portname = None   # Use default port

try:
    with mido.open_input(portname) as port:
        print('Using {}'.format(port))

        print("Ignoring everything but 'note_on' and 'note_off'.")
        print('Waiting for notes...')

        for message in accept_notes(port):
            print('Received {}'.format(message))
except KeyboardInterrupt:
    pass



"""
Meta messages for MIDI files.

Todo:
     - what if an unknown meta message is implemented and someone depends on
       the 'data' attribute?
     - is 'type_byte' a good name?
     - 'values' is not a good name for a dictionary.
     - type and value safety?
     - copy().
     - expose _key_signature_encode/decode?
"""
from __future__ import print_function, division
import sys
import math
import struct
from numbers import Integral
from contextlib import contextmanager
from ..messages import BaseMessage, check_time

PY2 = (sys.version_info.major == 2)

_charset = 'latin1'

def reverse_table(table):
    """Return value: key for dictionary."""
    return {value: key for (key, value) in table.items()}

_key_signature_decode = {
        (-7, 0): 'Cb',
        (-6, 0): 'Gb',
        (-5, 0): 'Db',
        (-4, 0): 'Ab',
        (-3, 0): 'Eb',
        (-2, 0): 'Bb',
        (-1, 0): 'F',
        (0, 0): 'C',
        (1, 0): 'G',
        (2, 0): 'D',
        (3, 0): 'A',
        (4, 0): 'E',
        (5, 0): 'B',
        (6, 0): 'F#',
        (7, 0): 'C#',
        (-7, 1): 'Abm',
        (-6, 1): 'Ebm',
        (-5, 1): 'Bbm',
        (-4, 1): 'Fm',
        (-3, 1): 'Cm',
        (-2, 1): 'Gm',
        (-1, 1): 'Dm',
        (0, 1): 'Am',
        (1, 1): 'Em',
        (2, 1): 'Bm',
        (3, 1): 'F#m',
        (4, 1): 'C#m',
        (5, 1): 'G#m',
        (6, 1): 'D#m',
        (7, 1): 'A#m',
    }
_key_signature_encode = reverse_table(_key_signature_decode)

_smpte_framerate_decode = {
        0: 24,
        1: 25,
        2: 29.97,
        3: 30,
    }
_smpte_framerate_encode = reverse_table(_smpte_framerate_decode)

def signed(to_type, n):
    formats = {
        'byte': 'Bb',
        'short': 'Hh',
        'long': 'Ll',
        'ubyte': 'bB',
        'ushort': 'hH',
        'ulong': 'lL',
        }

    try:
        pack_format, unpack_format = formats[to_type]
    except KeyError:
        raise ValueError('invalid integer type {}'.format(to_type))

    try:
        packed = struct.pack(pack_format, n)
        return struct.unpack(unpack_format, packed)[0]
    except struct.error as err:
        raise ValueError(*err.args)

def unsigned(to_type, n):
    return signed('u{}'.format(to_type), n)

def encode_variable_int(value):
    """Encode variable length integer.

    Returns the integer as a list of bytes,
    where the last byte is < 128.

    This is used for delta times and meta message payload
    length.
    """
    if not isinstance(value, Integral) or value < 0:
        raise ValueError('variable int must be a positive integer')

    bytes = []
    while value:
        bytes.append(value & 0x7f)
        value >>= 7

    if bytes:
        bytes.reverse()

        # Set high bit in every byte but the last.
        for i in range(len(bytes) - 1):
            bytes[i] |= 0x80
        return bytes
    else:
        return [0]

def encode_string(string):
    return list(bytearray(string.encode(_charset)))

def decode_string(data):
    return bytearray(data).decode(_charset)

def bpm2tempo(bpm):
    """Convert beats per minute to MIDI file tempo.

    Returns microseconds per beat as an integer::
    
        240 => 250000
        120 => 500000
        60 => 1000000
    """
    # One minute is 60 million microseconds.
    return int(round((60 * 1000000) / bpm))

def tempo2bpm(tempo):
    """Convert MIDI file tempo to BPM.

    Returns BPM as an integer or float::

        250000 => 240
        500000 => 120
        1000000 => 60
    """
    # One minute is 60 million microseconds.
    return (60 * 1000000) / tempo

@contextmanager
def meta_charset(tmp_charset):
    global _charset
    old = _charset
    _charset = tmp_charset
    yield
    _charset = old

def check_int(value, low, high):
    if not isinstance(value, int):
        raise TypeError('attribute must be an integer')
    elif not low <= value <= high:
        raise ValueError('attribute must be in range {}..{}'.format(low, high))

if PY2:
    def check_str(value):
        if not isinstance(value, basestring):
            raise TypeError('attribute must be unicode or string')
else:
    def check_str(value):
        if not isinstance(value, str):
            raise TypeError('attribute must a string')

class MetaSpec(object):
    def check(self, name, value):
        pass

class MetaSpec_sequence_number(MetaSpec):
    type_byte = 0x00
    attributes = ['number']
    defaults = [0]

    def decode(self, message, data):
        message.number = (data[0] << 8) | data[1]

    def encode(self, message):
        return [message.number >> 8, message.number & 0xff]

    def check(self, name, value):
        check_int(value, 0, 255)
            
class MetaSpec_text(MetaSpec):
    type_byte = 0x01
    attributes = ['text']
    defaults = ['']

    def decode(self, message, data):
        message.text = decode_string(data)

    def encode(self, message):
        return encode_string(message.text)

    def check(self, name, value):
        check_str(value)

class MetaSpec_copyright(MetaSpec_text):
    type_byte = 0x02

class MetaSpec_track_name(MetaSpec_text):
    type_byte = 0x03
    attributes = ['name']
    defaults = ['']
    
    def decode(self, message, data):
        message.name = decode_string(data)

    def encode(self, message):
        return encode_string(message.name)

class MetaSpec_instrument_name(MetaSpec_track_name):
    type_byte = 0x04

class MetaSpec_lyrics(MetaSpec_text):
    type_byte = 0x05

class MetaSpec_marker(MetaSpec_text):
    type_byte = 0x06

class MetaSpec_cue_marker(MetaSpec_text):
    type_byte = 0x07

class MetaSpec_device_name(MetaSpec_track_name):
    type_byte = 0x09

class MetaSpec_channel_prefix(MetaSpec):
    type_byte = 0x20
    attributes = ['channel']
    defaults = [0]

    def decode(self, message, data):
        message.channel = data[0]

    def encode(self, message):
        return [message.channel]

    def check(self, name, value):
        check_int(value, 0, 0xff)
            

class MetaSpec_midi_port(MetaSpec):
    type_byte = 0x21
    attributes = ['port']
    defaults = [0]

    def decode(self, message, data):
        message.port = data[0]

    def encode(self, message):
        return [message.port]

    def check(self, name, value):
        check_int(value, 0, 255)

class MetaSpec_end_of_track(MetaSpec):
    type_byte = 0x2f
    attributes = []
    defaults = []

    def decode(self, message, data):
        pass

    def encode(self, message):
        return []

class MetaSpec_set_tempo(MetaSpec):
    type_byte = 0x51
    attributes = ['tempo']
    defaults = [500000]

    def decode(self, message, data):
        message.tempo = (data[0] << 16) | (data[1] << 8) | (data[2])

    def encode(self, message):
        tempo = message.tempo
        return [tempo >> 16, tempo >> 8 & 0xff, tempo & 0xff]

    def check(self, name, value):
        check_int(value, 0, 0xffffff)

class MetaSpec_smpte_offset(MetaSpec):
    type_byte = 0x54
    attributes = ['frame_rate',
                  'hours',
                  'minutes',
                  'seconds',
                  'frames',
                  'sub_frames'
                ]
    # Todo: What are some good defaults?
    defaults = [24, 0, 0, 0, 0, 0]

    def decode(self, message, data):
        message.frame_rate = _smpte_framerate_decode[(data[0] >> 6)]
        message.hours = (data[0] & 0x3f)
        message.minutes = data[1]
        message.seconds = data[2]
        message.frames = data[3]
        message.sub_frames = data[4]

    def encode(self, message):
        frame_rate_lookup = _smpte_framerate_encode[message.frame_rate] << 6
        return [frame_rate_lookup | message.hours,
                message.minutes,
                message.seconds,
                message.frames,
                message.sub_frames]

    def check(self, name, value):
        if name == 'frame_rate':
            if value not in _smpte_framerate_encode:
                valid = ', '.join(sorted(_smpte_framerate_encode.keys()))
                raise ValueError('frame_rate must be one of {}'.format(valid))
        elif name == 'hours':
            check_int(value, 0, 255)
        elif name in ['minutes', 'seconds']:
            check_int(value, 0, 59)
        elif name == 'frames':
            check_int(value, 0, 255)
        elif name == 'sub_frames':
            check_int(value, 0, 99)

class MetaSpec_time_signature(MetaSpec):
    type_byte = 0x58
    # Todo: these need more sensible names.
    attributes = ['numerator',
                  'denominator',
                  'clocks_per_click',
                  'notated_32nd_notes_per_beat']
    defaults = [4, 2, 24, 8]

    def decode(self, message, data):
        message.numerator = data[0]
        message.denominator = 2 ** data[1]
        message.clocks_per_click = data[2]
        message.notated_32nd_notes_per_beat = data[3]

    def encode(self, message):
        return [
            message.numerator,
            int(math.log(message.denominator, 2)),
            message.clocks_per_click,
            message.notated_32nd_notes_per_beat,
            ]

    def check(self, name, value):
        if name == 'denominator':
            # This allows for the ridiculous time signature of
            # 4/57896044618658097711785492504343953926634...
            #   992332820282019728792003956564819968
            check_int(value, 1, 2 ** 255)
            encoded = math.log(value, 2)
            encoded_int = int(encoded)
            if encoded != encoded_int:
                raise ValueError('denominator must be a power of 2')
        else:
            check_int(value, 0, 255)

class MetaSpec_key_signature(MetaSpec):
    type_byte = 0x59
    attributes = ['key']
    defaults = ['C']

    def decode(self, message, data):
        key = signed('byte', data[0])
        mode = data[1]
        message.key = _key_signature_decode[(key, mode)]

    def encode(self, message):
        key, mode = _key_signature_encode[message.key]
        return [unsigned('byte', key), mode]

    def check(self, name, value):
        if not value in _key_signature_encode:
            raise ValueError('invalid key {!r}'.format(value))

class MetaSpec_sequencer_specific(MetaSpec):
    type_byte = 0x7f
    attributes = ['data']
    defaults = [[]]

    def decode(self, message, data):
        message.data = tuple(data)

    def encode(self, message):
        return list(message.data)

_specs = {}

def add_meta_spec(klass):
    spec = klass()
    if not hasattr(spec, 'type'):
        name = klass.__name__.replace('MetaSpec_', '')
        spec.type = name
    # This is used by copy().
    spec.settable_attributes = set(spec.attributes) | {'time'}
    _specs[spec.type_byte] = spec
    _specs[spec.type] = spec
    
def _add_builtin_meta_specs():
    for name in globals():
        if name.startswith('MetaSpec_'):
            add_meta_spec(globals()[name])

_add_builtin_meta_specs()


def build_meta_message(type_, data, delta=0):
    # Todo: handle unknown type.
    try:
        spec = _specs[type_]
    except KeyError:
        return UnknownMetaMessage(type_, data)

    message = MetaMessage(spec, time=delta)
    spec.decode(message, data)
    return message

class MetaMessage(BaseMessage):
    def __init__(self, type_, **kwargs):
        # Todo: allow type_ to be a type byte?
        # Todo: handle unknown type.
        if isinstance(type_, MetaSpec):
            vars(self)['_spec'] = type_
        else:
            vars(self)['_spec'] = _specs[type_]

        vars(self)['type'] = self._spec.type

        for name in kwargs:
            if name == 'time':
                continue  # Time is always allowed.

            if name not in self._spec.settable_attributes:
                raise ValueError(
                    '{} is not a valid argument for this message type'.format(
                        name))

        for name, value in zip(self._spec.attributes, self._spec.defaults):
            vars(self)[name] = value
        vars(self)['time'] = 0

        for name, value in kwargs.items():
            setattr(self, name, value)

    def __setattr__(self, name, value):
        if name in self._spec.settable_attributes:
            if name == 'time':
                check_time(value)
            else:
                self._spec.check(name, value)
            vars(self)[name] = value
        elif name in vars(self):
            raise AttributeError('{} attribute is read only'.format(name))
        else:
            raise AttributeError(
                '{} message has no attribute {}'.format(self.type, name))

    def bytes(self):
        data = self._spec.encode(self)
        return ([0xff, self._spec.type_byte]
                + encode_variable_int(len(data))
                + data)
    
    def __repr__(self):
        attributes = []
        for name in self._spec.attributes:
            attributes.append('{}={!r}'.format(name, getattr(self, name)))
        attributes = ' '.join(attributes)
        if attributes:
            attributes = (' {}'.format(attributes))

        return '<meta message {}{} time={}>'.format(self.type,
                                                    attributes, self.time)

class UnknownMetaMessage(MetaMessage):
    def __init__(self, type_byte, data=None, time=0):
        if data is None:
            data = []

        self.type = 'unknown_meta'
        self._type_byte = type_byte
        self._data = data
        self.time = time

    def __repr__(self):
        return ('<unknown meta message'
                ' _type_byte=0x{:02x} _data={!r} time={}>').format(
                self._type_byte,
                self._data,
                self.time)

    # Override all checking.
    def __setattr__(self, name, value):
        vars(self)[name] = value

    def bytes(self):
        return ([0xff, self._type_byte]
                + encode_variable_int(len(self._data))
                + self._data)



import io

from .messages import Message
from .midifiles import tempo2bpm, bpm2tempo
from .midifiles import MidiTrack, MetaMessage

def test_tempo2bpm_bpm2tempo():
    for bpm, tempo in [
            (20, 3000000),
            (60, 1000000),
            (120, 500000),
            (240, 250000),
    ]:
        assert bpm == tempo2bpm(tempo)
        assert tempo == bpm2tempo(bpm)


def test_track_slice():
    track = MidiTrack()

    # Slice should return MidiTrack object.
    assert isinstance(track[::], MidiTrack)


def test_track_name():
    name1 = MetaMessage('track_name', name='name1')
    name2 = MetaMessage('track_name', name='name2')

    # The track should use the first name it finds.
    track = MidiTrack([name1, name2])
    assert track.name == name1.name



"""
Useful tools for working with ports

Module content:

    multi_receive -- receive messages from multiple ports
    IOPort -- combined input / output port. Wraps around to normal ports
    MessageBuffer -- pseudo-port that stores messages in a deque
"""

from __future__ import unicode_literals
import threading
import time
import random
from collections import deque
from .parser import Parser
from .messages import Message

# How many seconds to sleep before polling again.
_DEFAULT_SLEEP_TIME = 0.001
_sleep_time = _DEFAULT_SLEEP_TIME


# Todo: document this more.
def sleep():
    """Sleep for N seconds.

    This is used in ports when polling and waiting for messages. N can
    be set with set_sleep_time()."""
    time.sleep(_sleep_time)


def set_sleep_time(seconds=_DEFAULT_SLEEP_TIME):
    """Set the number of seconds sleep() will sleep."""
    global _sleep_time
    _sleep_time = seconds


def get_sleep_time():
    """Get number of seconds sleep() will sleep."""
    return _sleep_time


class BasePort(object):
    """
    Abstract base class for Input and Output ports.
    """

    def __init__(self, name=None, **kwargs):
        self.name = name
        self._lock = threading.RLock()
        self.closed = True
        self._open(**kwargs)
        self.closed = False
 
    def _open(self, **kwargs):
        pass

    def _close(self):
        pass

    def close(self):
        """Close the port.

        If the port is already closed, nothing will happen.  The port
        is automatically closed when the object goes out of scope or
        is garbage collected.
        """
        with self._lock:
            if not self.closed:
                if hasattr(self, 'autoreset') and self.autoreset:
                    try:
                        self.reset()
                    except IOError:
                        pass

                self._close()
                self.closed = True

    def __del__(self):
        self.close()

    def __enter__(self):
        return self

    def __exit__(self, type, value, traceback):
        self.close()
        return False

    def __repr__(self):
        if self.closed:
            state = 'closed'
        else:
            state = 'open'

        capabilities = (hasattr(self, 'receive'), hasattr(self, 'send'))
        port_type = {
            (True, False): 'input',
            (False, True): 'output',
            (True, True): 'I/O',
            (False, False): 'mute',
            }[capabilities]

        name = self.name or ''

        try:
            device_type = self._device_type
        except AttributeError:
            device_type = self.__class__.__name__

        return '<{} {} {!r} ({})>'.format(
            state, port_type, name, device_type)


class BaseInput(BasePort):
    """Base class for input port.

    Subclass and override _receive() to create a new input port type.
    (See portmidi.py for an example of how to do this.)
    """
    def __init__(self, name='', **kwargs):
        """Create an input port.

        name is the port name, as returned by input_names(). If
        name is not passed, the default input is used instead.
        """
        BasePort.__init__(self, name, **kwargs)
        self._parser = Parser()
        self._messages = self._parser.messages  # Shortcut.


    def _check_callback(self):
        if hasattr(self, 'callback') and self.callback is not None:
            raise ValueError('a callback is set for this port')

    def _receive(self, block=True):
        pass

    def pending(self):
        """Return how many messages are ready to be received.

        *Note*: This is kept around for backward compatibility.  It's
        better to use iter_pending() to iterate over pending messages.

        This will read data from the device and put it in the
        parser. I will then return the number of messages available to
        be received.

        The value will not be reliable when receiving messages in
        multiple threads.

        If this is called on a closed port it will work as normal
        except it won't try to read from the device.
        """
        with self._lock:
            if not self.closed:
                self._check_callback()
                self._receive(block=False)

            return len(self._messages)

    def iter_pending(self):
        """Iterate through pending messages."""
        while True:
            message = self.poll()
            if message is None:
                return
            else:
                yield message

    def receive(self, block=True):
        """Return the next message.

        This will block until a message arrives.

        If you pass block=False it will not block and instead return
        None if there is no available message.

        If the port is closed and there are no pending messages IOError
        will be raised. If the port closes while waiting inside receive(),
        IOError will be raised. Todo: this seems a bit inconsistent. Should
        different errors be raised? What's most useful here?
        """
        self._check_callback()

        # If there is a message pending, return it right away.
        with self._lock:
            if self._messages:
                return self._messages.popleft()

        if self.closed:
            if block:
                raise ValueError('receive() called on closed port')
            else:
                return None

        while True:
            with self._lock:
                # In the future we should allow self._receive() to
                # return a message.  This would make the API more
                # consistent and also make it easier to implement
                # thread safe ports since self._messages is no longer
                # required.
                #
                # This can only be done if pending() is removed, since
                # it calls self._receive().
                #
                # A commented out implementation is provided below.
                self._receive(block=block)
                # msg = self._receive(block=block)
                # if msg:
                #     return msg
                # # (else check self._messages as before.)

                if self._messages:
                    return self._messages.popleft()
                elif not block:
                    return None
                elif self.closed:
                    raise IOError('port closed during receive()')

            sleep()

    def poll(self):
        """Receive the next pending message or None

        This is the same as calling `receive(block=False)`."""
        return self.receive(block=False)

    def __iter__(self):
        """Iterate through messages until the port closes."""
        # This could have simply called receive() in a loop, but that
        # could result in a "port closed during receive()" error which
        # is hard to catch here.
        self._check_callback()
        while True:
            try:
                yield self.receive()
            except IOError:
                if self.closed:
                    # The port closed before or inside receive().
                    # (This makes the assumption that this is the reason,
                    # at the risk of masking other errors.)
                    return
                else:
                    raise


class BaseOutput(BasePort):
    """
    Base class for output port.

    Subclass and override _send() to create a new port type.  (See
    portmidi.py for how to do this.)
    """

    def __init__(self, name='', autoreset=False, **kwargs):
        """Create an output port

        name is the port name, as returned by output_names(). If
        name is not passed, the default output is used instead.
        """
        BasePort.__init__(self, name, **kwargs)
        self.autoreset = autoreset
 
    def _send(self, message):
        pass

    def send(self, message):
        """Send a message on the port.

        A copy of the message will be sent, so you can safely modify
        the original message without any unexpected consequences.
        """
        if not isinstance(message, Message):
            raise TypeError('argument to send() must be a Message')
        elif self.closed:
            raise ValueError('send() called on closed port')

        with self._lock:
            self._send(message.copy())

    def reset(self):
        """Send "All Notes Off" and "Reset All Controllers" on all channels
        """
        if self.closed:
            return

        ALL_NOTES_OFF = 123
        RESET_ALL_CONTROLLERS = 121
        for channel in range(16):
            for control in [ALL_NOTES_OFF, RESET_ALL_CONTROLLERS]:
                self.send(Message('control_change',
                                  channel=channel,
                                  control=control))

    def panic(self):
        """Send "All Sounds Off" on all channels.

        This will mute all sounding notes regardless of
        envelopes. Useful when notes are hanging and nothing else
        helps.
        """
        if self.closed:
            return

        ALL_SOUNDS_OFF = 120
        for channel in range(16):
            self.send(Message('control_change',
                              channel=channel,
                              control=ALL_SOUNDS_OFF))


class BaseIOPort(BaseInput, BaseOutput):
    def __init__(self, name='', **kwargs):
        """Create an IO port.

        name is the port name, as returned by ioport_names().
        """
        BaseInput.__init__(self, name, **kwargs)
        BaseOutput.__init__(self, name, **kwargs)


class IOPort(BaseIOPort):
    """Input / output port.

    This is a convenient wrapper around an input port and an output
    port which provides the functionality of both. Every method call
    is forwarded to the appropriate port.
    """

    def __init__(self, input, output):
        self.input = input
        self.output = output

        # We use str() here in case name is None.
        self.name = '{} + {}'.format(str(input.name), str(output.name))
        self._messages = self.input._messages
        self.closed = False
        self._lock = threading.RLock()

    def _close(self):
        self.input.close()
        self.output.close()

    def _send(self, message):
        self.output._send(message)

    def _receive(self, block=True):
        return self.input._receive()


class EchoPort(BaseIOPort):
    def _send(self, message):
        self._messages.append(message)

    __iter__ = BaseIOPort.iter_pending


class MultiPort(BaseIOPort):
    def __init__(self, ports, yield_ports=False):
        BaseIOPort.__init__(self, 'multi')
        self.ports = list(ports)
        self.yield_ports = yield_ports

    def _send(self, message):
        for port in self.ports:
            if not port.closed:
                # Todo: what if a SocketPort connection closes in-between here?
                port.send(message)

    def _receive(self, block=True):
        self._messages.extend(multi_receive(self.ports,
                                            yield_ports=self.yield_ports,
                                            block=False))


def multi_receive(ports, yield_ports=False, block=True):
    """Receive messages from multiple ports.

    Generates messages from ever input port. The ports are polled in
    random order for fairness, and all messages from each port are
    yielded before moving on to the next port.

    If yield_ports=True, (port, message) is yielded instead of just
    the message.

    If block=False only pending messages will be yielded.
    """
    while True:
        # Make a shuffled copy of the port list.
        ports = list(ports)
        random.shuffle(ports)

        for port in ports:
            if not port.closed:
                for message in port.iter_pending():
                    if yield_ports:
                        yield port, message
                    else:
                        yield message

        if block:
            sleep()
        else:
            break


def multi_iter_pending(ports, yield_ports=False):
    """Iterate through all pending messages in ports.

    This is the same as calling multi_receive(ports, block=False).
    The function is kept around for backwards compatability.
    """
    return multi_receive(ports, yield_ports=yield_ports, block=False)



#!/usr/bin/env python
"""
Create a new MIDI file with some random notes.

The file is saved to test.mid.
"""

import random
from mido import Message, MidiFile, MidiTrack

notes = [64, 64+7, 64+12]

outfile = MidiFile()

track = MidiTrack()
outfile.tracks.append(track)

track.append(Message('program_change', program=12))

delta = 16
for i in range(4):
    note = random.choice(notes)
    track.append(Message('note_on', note=note, velocity=100, time=delta))
    track.append(Message('note_off', note=note, velocity=100, time=delta))
    
outfile.save('test.mid')




"""
Backend for python-rtmidi:

http://pypi.python.org/pypi/python-rtmidi/
"""
from __future__ import absolute_import
import os
import sys
import time

PY2 = (sys.version_info.major == 2)
if PY2:
    import Queue as queue
else:
    import queue

import rtmidi
from ..ports import get_sleep_time, BaseInput, BaseOutput


def _get_api_lookup():
    api_to_name = {}
    name_to_api = {}

    for name in dir(rtmidi):
        if name.startswith('API_'):
            value = getattr(rtmidi, name)
            name = name.replace('API_', '')
            name_to_api[name] = value
            api_to_name[value] = name

    return api_to_name, name_to_api

_api_to_name, _name_to_api = _get_api_lookup()

def _get_api_id(name=None):
    if name is None:
        return rtmidi.API_UNSPECIFIED

    try:
        api = _name_to_api[name]
    except KeyError:
        raise ValueError('unknown API {}'.format(name))    

    if name in get_api_names():
        return api
    else:
        raise ValueError('API {} not compiled in'.format(name))

def get_devices(api=None, **kwargs):
    devices = []

    rtapi = _get_api_id(api)
    input_names = set(rtmidi.MidiIn(rtapi=rtapi).get_ports())
    output_names = set(rtmidi.MidiOut(rtapi=rtapi).get_ports())

    for name in sorted(input_names | output_names):
        devices.append({
                'name': name,
                'is_input': name in input_names,
                'is_output': name in output_names,
                })

    return devices

def get_api_names():
    return [_api_to_name[n] for n in rtmidi.get_compiled_api()]

class PortCommon(object):
    def _open(self, api=None, virtual=False, **kwargs):

        rtapi = _get_api_id(api)
        opening_input = hasattr(self, 'receive')

        if opening_input:
            self._rt = rtmidi.MidiIn(rtapi=rtapi)
            self._rt.ignore_types(False, False, True)
            self._queue = queue.Queue()

            callback = kwargs.get('callback')
            if callback is not None:
                self.callback = kwargs.get('callback')
        else:
            self._rt = rtmidi.MidiOut(rtapi=rtapi)
            # Turn of ignore of sysex, time and active_sensing.

        ports = self._rt.get_ports()

        if virtual:
            if self.name is None:
                raise IOError('virtual port must have a name')
            self._rt.open_virtual_port(self.name)
        else:
            if self.name is None:
                # Todo: this could fail if list is empty.
                # In RtMidi, the default port is the first port.
                try:
                    self.name = ports[0]
                except IndexError:
                    raise IOError('no ports available')

            try:
                port_id = ports.index(self.name)
            except ValueError:
                raise IOError('unknown port {!r}'.format(self.name))

            try:
                self._rt.open_port(port_id)
            except RuntimeError as err:
                raise IOError(*err.args)

        api = _api_to_name[self._rt.get_current_api()]
        self._device_type = 'RtMidi/{}'.format(api)
        if virtual:
            self._device_type = 'virtual {}'.format(self._device_type)

    def _close(self):
        self._rt.close_port()
        del self._rt  # Virtual ports are closed when this is deleted.

class Input(PortCommon, BaseInput):
    @property
    def callback(self):
        return self._callback

    @callback.setter
    def callback(self, func):
        self._rt.cancel_callback()

        if func:
            # First send all queued messages to the callback.
            while True:
                try:
                    func(self._queue.get_nowait())
                except queue.Empty:
                    break

        if func is None:
            self._rt.set_callback(self._feed_queue)
            self._callback = None
        else:
            self._rt.set_callback(self._feed_callback)
            self._callback = func

    def _feed_queue(self, message_data, data):
        self._parser.feed(message_data[0])
        for message in self._parser:
            self._queue.put(message)

    def _feed_callback(self, message_data, data):
        self._parser.feed(message_data[0])
        for message in self._parser:
            self._callback(message)

    # receive() implementation:
    #
    # We need to override receive() here because _receive() doesn't allow
    # us to return a messages directly and putting it in self._messages is
    # not an option.
    #
    # Todo: add doc strings.

    if PY2:
        # In Python 2 queue.get() doesn't respond to CTRL-C. A workaroud is
        # to call queue.get(timeout=100) (very high timeout) in a loop, but all
        # that does is poll with a timeout of 50 milliseconds. This results in
        # much too high latency.
        #
        # It's better to do our own polling with a shorter sleep time.
        #
        # See Issue #49 and https://bugs.python.org/issue8844

        def receive(self, block=True):
            sleep_time = get_sleep_time()
            while True:
                try:
                    return self._queue.get_nowait()
                except queue.Empty:
                    if block:
                        time.sleep(sleep_time)
                        continue
                    else:
                        return None
    else:

        def receive(self, block=True):
            try:
                return self._queue.get(block=block)
            except queue.Empty:
                return None

    receive.__doc__ = BaseInput.receive.__doc__

    def pending(self):
        return self._queue.qsize()

class Output(PortCommon, BaseOutput):
    def _send(self, message):
        self._rt.send_message(message.bytes())



from pytest import raises
from .messages import Message
from .syx import read_syx_file, write_syx_file

def test_read(tmpdir):
    # p = tmpdir.mkdir("sub").join("hello.txt")
    path = tmpdir.join("test.syx").strpath

    message = Message('sysex', data=(1, 2, 3))

    with open(path, 'wb') as outfile:
        outfile.write(message.bin())
    assert read_syx_file(path) == [message]

    with open(path, 'wt') as outfile:
        outfile.write(message.hex())
    assert read_syx_file(path) == [message]

    with open(path, 'wt') as outfile:
        outfile.write('NOT HEX')
    with raises(ValueError):
        read_syx_file(path)

def test_write(tmpdir):
    # p = tmpdir.mkdir("sub").join("hello.txt")
    path = tmpdir.join("test.syx").strpath

    message = Message('sysex', data=(1, 2, 3))

    write_syx_file(path, [message])
    with open(path, 'rb') as infile:
        assert infile.read() == message.bin()

    write_syx_file(path, [message], plaintext=True)
    with open(path, 'rt') as infile:
        assert infile.read().strip() == message.hex()



#!/usr/bin/env python
"""
Send random notes to the output port.
"""

from __future__ import print_function
import sys
import time
import random
import mido
from mido import Message


if len(sys.argv) > 1:
    portname = sys.argv[1]
else:
    portname = None  # Use default port

# A pentatonic scale
notes = [60, 62, 64, 67, 69, 72]

try:
    with mido.open_output(portname, autoreset=True) as port:
        print('Using {}'.format(port))
        while True:
            note = random.choice(notes)

            on = Message('note_on', note=note)
            print('Sending {}'.format(on))
            port.send(on)
            time.sleep(0.05)

            off = Message('note_off', note=note)
            print('Sending {}'.format(off))
            port.send(off)
            time.sleep(0.1)
except KeyboardInterrupt:
    pass

print()



from pytest import raises
from .sockets import parse_address

def test_parse_address():
    assert parse_address(':8080') == ('', 8080)
    assert parse_address('localhost:8080') == ('localhost', 8080)

    with raises(ValueError): parse_address(':to_many_colons:8080')
    with raises(ValueError): parse_address('only_hostname')
    with raises(ValueError): parse_address('')
    with raises(ValueError): parse_address(':')
    with raises(ValueError): parse_address(':shoe')
    with raises(ValueError): parse_address(':0')
    with raises(ValueError): parse_address(':65536')  # Out of range.



"""
Read and write SYX file format
"""
from __future__ import print_function
import re
import string
import binascii
from .messages import Message
from .parser import Parser, parse_all

_HEXDIGITS = set(string.hexdigits.encode('ascii'))

def read_syx_file(filename):
    """Read sysex messages from SYX file.

    Returns a list of sysex messages.
    
    This handles both the text (hexadecimal) and binary
    formats. Messages other than sysex will be ignored. Raises
    ValueError if file is plain text and byte is not a 2-digit hex
    number.
    """
    def raise_value_error():
        raise ValueError('{!r} line {}: invalid hex byte {!r}'.format(
            filename, lineno, byte))

    with open(filename, 'rb') as infile:
        data = infile.read()

    if len(data) == 0:
        # Empty file.
        return []

    # data[0] will give a byte string in Python 2 and an integer in
    # Python 3.
    if data[0] in (b'\xf0', 240):
        # Binary.
        return parse_all(data)
    else:
        # Hex.
        # We're decoding as latin1 to avoid decoding errors.
        data = bytearray.fromhex(data.decode('latin1'))

        return [msg for msg in Parser(data) if msg.type == 'sysex']


def write_syx_file(filename, messages, plaintext=False):
    """Write sysex messages to a SYX file.

    Messages other than sysex will be skipped.

    By default this will write the binary format.  Pass
    ``plaintext=True`` to write the plain text format (hex encoded
    ASCII text).
    """
    messages = [m for m in messages if m.type == 'sysex']

    if plaintext:
        with open(filename, 'wt') as outfile:
            for message in messages:
                outfile.write(message.hex())
                outfile.write('\n')
    else:
        with open(filename, 'wb') as outfile:
            for message in messages:
                outfile.write(message.bin())



#!/usr/bin/env python
"""
Queue port for threading.
"""

import sys
PY2 = (sys.version_info.major == 2)

if PY2:
    from Queue import Queue, Empty
else:
    from queue import Queue, Empty
from threading import Thread
import time
import random

from mido import Message
from mido.ports import BaseInput, BaseOutput

PY2 = (sys.version_info.major == 2)

class QueuePort(BaseInput, BaseOutput):
    def _open(self, **kwargs):
        maxsize = kwargs.get('maxsize', 0)
        self._queue = Queue(maxsize)

    def _send(self, message):
        self._queue.put(message)

    # We need to override receive() since _receive() is not
    # allowed to return a message directly.
    def receive(self, block=True):
        try:
            return self._queue.get(block=block)
        except Empty:
            return None
    
    def _get_device_type(self):
        return 'queue'
    
if __name__ == '__main__':
    def program_changer():
        while True:
            queue_port.send(
                Message('program_change', program=random.randrange(128)))
            time.sleep(0.2)

    queue_port = QueuePort(maxsize=3)

    thread = Thread(target=program_changer)
    thread.daemon = True
    thread.start()

    for message in queue_port:
        print(message)



"""
Try out the new printer port backend.

It also works with MIDO_BACKEND, so you can do:

    $ MIDO_BACKEND=printer python
    >>> import mido
    >>> mido.get_output_names()
    ['The Printer Port']
"""

import time
import mido

mido.set_backend('printer')

print('Available outputs: {!r}'.format(mido.get_output_names()))

with mido.open_output() as port:
    print('Using {}.'.format(port))

    port.send(mido.Message('program_change', program=10))
    for i in [1, 2, 3]:
        port.send(mido.Message('control_change', control=1, value=i))



"""
MIDI Parser

There is no need to use this module directly. All you need is
available in the top level module.
"""

#
# About running status:
#
# http://home.roadrunner.com/~jgglatt/tech/midispec/run.htm
# http://stackoverflow.com/questions/3660964/get-note-data-from-midi-file
#

import sys
from collections import deque
from .messages import Message, build_message, get_spec

PY2 = (sys.version_info.major == 2)


class Parser(object):
    """
    MIDI Parser

    Parses a stream of bytes and produces messages.

    Data can be put into the parser in the form of
    integers, byte arrays or byte strings.
    """

    def __init__(self, data=None):
        """Create a new parser."""
        self.messages = deque()
        self._reset()
        if data is not None:
            self.feed(data)

    def _reset(self):
        self._spec = None
        self._bytes = []
        self._inside_sysex = False

    def _deliver(self, message=None):
        if not message:
            message = build_message(self._spec, self._bytes)
        self.messages.append(message)

    def _handle_status_byte(self, byte):
        try:
            spec = get_spec(byte)
            valid = True
        except LookupError:
            valid = False

        if self._inside_sysex and (0xf8 <= byte <= 0xff):
            # Realtime message inside sysex.
            # Deliver it straight away.
            if valid:
                message = build_message(spec, [byte])
                self._deliver(message)
        elif byte == 0xf7:
            # End of sysex.
            if self._inside_sysex:
                self._deliver()
            self._reset()
        elif valid:
            # Start new message.
            self._spec = spec
            self._bytes = [byte]
            self._inside_sysex = (byte == 0xf0)
        else:
            # Ignore message.
            self._reset()

    def feed_byte(self, byte):
        """Feed one MIDI byte into the parser.

        The byte must be an integer in range 0..255.
        """
        try:
            int(byte)
        except TypeError:
            raise TypeError('byte must be an integer')

        if not 0 <= byte <= 255:
            raise ValueError('byte must be in range 0..255')

        if byte >= 0x80:
            self._handle_status_byte(byte)
        else:
            # Data byte.
            self._bytes.append(byte)

        # If we have a complete messages, deliver it.
        if self._spec and len(self._bytes) == self._spec.length:
            self._deliver()
            self._reset()

    def feed(self, data):
        """Feed MIDI data to the parser.

        Accepts any object that produces a sequence of integers in
        range 0..255, such as:

            [0, 1, 2]
            (0, 1, 2)
            [for i in range(256)]
            (for i in range(256)]
            bytearray()
            b''  # Will be converted to integers in Python 2.
        """
        if PY2 and isinstance(data, str):
            # Byte strings in Python 2 need extra attention.
            for char in data:
                self.feed_byte(ord(char))
        else:
            for byte in data:
                self.feed_byte(byte)

    def pending(self):
        """Return the number of pending messages."""
        return len(self.messages)

    def get_message(self):
        """Get the first parsed message.

        Returns None if there is no message yet. If you don't want to
        deal with None, you can use pending() to see how many messages
        you can get before you get None.
        """
        if self.messages:
            return self.messages.popleft()
        else:
            return None

    def __iter__(self):
        """Yield messages that have been parsed so far."""
        while len(self.messages):
            yield self.messages.popleft()


def parse_all(data):
    """Parse MIDI data and return a list of all messages found.

    This is typically used to parse a little bit of data with a few
    messages in it. It's best to use a Parser object for larger
    amounts of data. Also, tt's often easier to use parse() if you
    know there is only one message in the data.
    """
    return list(Parser(data))


def parse(data):
    """ Parse MIDI data and return the first message found.

    Data after the first message is ignored. Use parse_all()
    to parse more than one message.
    """
    return Parser(data).get_message()



#!/usr/bin/env python
"""
List available PortMidi ports.
"""

from __future__ import print_function
import sys
import mido

def print_ports(heading, port_names):
    print(heading)
    for name in port_names:
        print("    '{}'".format(name))
    print()

print()
print_ports('Input Ports:', mido.get_input_names())
print_ports('Output Ports:', mido.get_output_names())



#!/usr/bin/env python
import os
import sys
import mido

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

if sys.argv[-1] == "publish":
    os.system("python setup.py sdist upload")
    sys.exit()

elif sys.argv[-1] == "docs":
    os.system("sphinx-build docs docs/_build")
    sys.exit()

setup(
    name='mido',
    version=mido.__version__,
    description='MIDI Objects for Python',
    long_description=open('README.rst', 'rt').read(),
    author=mido.__author__,
    author_email=mido.__email__,
    url=mido.__url__,
    package_data={'': ['LICENSE']},
    package_dir={'mido': 'mido'},
    packages = ['mido', 'mido.backends'],
    scripts = ['bin/mido-play',
               'bin/mido-ports',
               'bin/mido-serve',
               'bin/mido-connect'],
    include_package_data=True,
    install_requires=[],
    license='MIT',
    zip_safe=False,
)



"""
MIDI file reading and playback.

References:

http://home.roadrunner.com/~jgglatt/
http://home.roadrunner.com/~jgglatt/tech/miditech.htm
http://home.roadrunner.com/~jgglatt/tech/midifile.htm

http://www.sonicspot.com/guide/midifiles.html
http://www.ccarh.org/courses/253/assignment/midifile/
https://code.google.com/p/binasc/wiki/mainpage
http://stackoverflow.com/questions/2984608/midi-delta-time
http://www.recordingblogs.com/sa/tabid/82/EntryId/44/MIDI-Part-XIII-Delta-time-a
http://www.sonicspot.com/guide/midifiles.html
"""

from __future__ import print_function, division
import io
import sys
import time
import string
import struct
from ..messages import build_message, Message, get_spec
from .meta import MetaMessage, build_meta_message, meta_charset
from .meta import MetaSpec, add_meta_spec, encode_variable_int
from .tracks import MidiTrack, merge_tracks, fix_end_of_track

PY2 = (sys.version_info.major == 2)

# The default tempo is 120 BPM.
# (500000 microseconds per beat (quarter note).)
DEFAULT_TEMPO = 500000
DEFAULT_TICKS_PER_BEAT = 480

def print_byte(byte, pos=0):
    char = chr(byte)
    if char.isspace() or not char in string.printable:
        char = '.'

    print('  {:06x}: {:02x}  {}'.format(pos, byte, char))


class DebugFileWrapper(object):
    def __init__(self, file):
        self.file = file

    def read(self, size):
        data = self.file.read(size)

        for byte in data:
            print_byte(byte, self.file.tell())

        return data

    def tell(self):
        return self.file.tell()


def read_byte(self):
    byte = self.read(1)
    if byte == b'':
        raise EOFError
    else:
        return ord(byte)


def read_bytes(infile, size):
    return [read_byte(infile) for _ in range(size)]


def _dbg(text=''):
    print(text)


# We can't use the chunk module for two reasons:
#
# 1. we may have mixed big and little endian chunk sizes. (RIFF is
# little endian while MTrk is big endian.)
#
# 2. the chunk module assumes that chunks are padded to the neares
# multiple of 2. This is not true of MIDI files.

def read_chunk_header(infile):
    header = infile.read(8)
    if len(header) < 8:
        raise EOFError

    # Todo: check for b'RIFF' and switch endian?

    return struct.unpack('>4sL', header)


def read_file_header(infile):
    name, size = read_chunk_header(infile)

    if name != b'MThd':
        raise IOError('MThd not found. Probably not a MIDI file')
    else:
        data = infile.read(size)

        if len(data) < 6:
            raise EOFError

        return struct.unpack('>hhh', data[:6])


def read_message(infile, status_byte, peek_data, delta):
    try:
        spec = get_spec(status_byte)
    except LookupError:
        raise IOError('undefined status byte 0x{:02x}'.format(status_byte))

    # Subtrac 1 for status byte.
    size = spec.length - 1 - len(peek_data)
    data_bytes = peek_data + read_bytes(infile, size)

    for byte in data_bytes:
        if byte > 127:
            raise IOError('data byte must be in range 0..127')
    return build_message(spec, [status_byte] + data_bytes, time=delta)


def read_sysex(infile, delta):
    length = read_variable_int(infile)
    data = read_bytes(infile, length)

    # Strip start and end bytes.
    # Todo: is this necessary?
    if data and data[0] == 0xf0:
        data = data[1:]
    if data and data[-1] == 0xf7:
        data = data[:-1]

    return Message('sysex', data=data, time=delta)


def read_variable_int(infile):
    delta = 0

    while True:
        byte = read_byte(infile)
        delta = (delta << 7) | (byte & 0x7f)
        if byte < 0x80:
            return delta


def read_meta_message(infile, delta):
    type = read_byte(infile)
    length = read_variable_int(infile)
    data = read_bytes(infile, length)
    return build_meta_message(type, data, delta)


def read_track(infile, debug=False):
    track = MidiTrack()

    name, size = read_chunk_header(infile)

    if name != b'MTrk':
        raise IOError('no MTrk header at start of track')

    if debug:
        _dbg('-> size={}'.format(size))
        _dbg()

    start = infile.tell()
    last_status = None

    while True:
        # End of track reached.
        if infile.tell() - start == size:
            break

        if debug:
            _dbg('Message:')

        delta = read_variable_int(infile)

        if debug:
            _dbg('-> delta={}'.format(delta))

        # Todo: not all messages have running status
        status_byte = read_byte(infile)

        if status_byte < 0x80:
            if last_status is None:
                raise IOError('running status without last_status')
            peek_data = [status_byte]
            status_byte = last_status
        else:
            if status_byte != 0xff:
                # Meta messages don't set running status.
                last_status = status_byte
            peek_data = []

        if status_byte == 0xff:
            msg = read_meta_message(infile, delta)
        elif status_byte in [0xf0, 0xf7]:
            # Todo: I'm not quite clear on the difference between
            # f0 and f7 events.
            msg = read_sysex(infile, delta)
        else:
            msg = read_message(infile, status_byte, peek_data, delta)

        track.append(msg)

        if debug:
            _dbg('-> {!r}'.format(msg))
            _dbg()

    return track


def write_chunk(outfile, name, data):
    """Write an IFF chunk to the file.

    `name` must be a bytestring."""
    outfile.write(name)
    outfile.write(struct.pack('>L', len(data)))
    outfile.write(data)


def write_track(outfile, track):
    data = bytearray()

    running_status_byte = None
    for msg in fix_end_of_track(track):
        if not isinstance(msg, MetaMessage):
            if msg._spec.status_byte >= 0xf8:
                raise ValueError(
                    ("realtime message '{}' is not "
                     "allowed in a MIDI file".format(
                            message.type)))

        data.extend(encode_variable_int(msg.time))
        if msg.type == 'sysex':
            running_status_byte = None
            data.append(0xf0)
            # length (+ 1 for end byte (0xf7))
            data.extend(encode_variable_int(len(msg.data) + 1))
            data.extend(msg.data)
            data.append(0xf7)
        else:
            raw = msg.bytes()
            if (isinstance(msg, Message)
                and raw[0] < 0xf0
                and raw[0] == running_status_byte):
                data.extend(raw[1:])
            else:
                data.extend(raw)
            running_status_byte = raw[0]

    write_chunk(outfile, b'MTrk', data)



class MidiFile:
    def __init__(self, filename=None, file=None,
                 type=1, ticks_per_beat=DEFAULT_TICKS_PER_BEAT,
                 charset='latin1',
                 debug=False):

        self.filename = filename
        self.type = type
        self.ticks_per_beat = ticks_per_beat
        self.charset = charset
        self.debug = debug

        self.tracks = []

        if type not in range(3):
            raise ValueError(
                'invalid format {} (must be 0, 1 or 2)'.format(format))

        if file is not None:
            self._load(file)
        elif self.filename is not None:
            with io.open(filename, 'rb') as file:
                self._load(file)

    def add_track(self, name=None):
        """Add a new track to the file.

        This will create a new MidiTrack object and append it to the
        track list.
        """
        track = MidiTrack()
        if name is not None:
            track.name = name
        self.tracks.append(track)
        return track

    def _load(self, infile):
        if self.debug:
            infile = DebugFileWrapper(infile)

        with meta_charset(self.charset):
            if self.debug:
                _dbg('Header:')

            (self.type,
             num_tracks,
             self.ticks_per_beat) = read_file_header(infile)

            if self.debug:
                _dbg('-> type={}, tracks={}, ticks_per_beat={}'.format(
                    self.type, num_tracks, self.ticks_per_beat))
                _dbg()

            for i in range(num_tracks):
                if self.debug:
                    _dbg('Track {}:'.format(i))

                self.tracks.append(read_track(infile, debug=self.debug))
                # Todo: used to ignore EOFError. I hope things still work.

    @property
    def length(self):
        """Playback time in seconds.

        This will be computed by going through every message in every
        track and adding up delta times.
        """
        if self.type == 2:
            raise ValueError('impossible to compute length'
                             ' for type 2 (asynchronous) file')

        return sum(msg.time for msg in self)

    def _get_seconds_per_tick(self, tempo):
        # Tempo is given in microseconds per beat (default 500000).
        # At this tempo there are (500000 / 1000000) == 0.5 seconds
        # per beat. At the default resolution of 480 ticks per beat
        # this is:
        #
        #    (500000 / 1000000) / 480 == 0.5 / 480 == 0.0010417
        #
        return (tempo / 1000000.0) / self.ticks_per_beat

    def __iter__(self):
        # The tracks of type 2 files are not in sync, so they can
        # not be played back like this.
        if self.type == 2:
            raise TypeError("can't merge tracks in type 2 (asynchronous) file")

        seconds_per_tick = self._get_seconds_per_tick(DEFAULT_TEMPO)

        for msg in merge_tracks(self.tracks):
            # Convert message time from absolute time
            # in ticks to relative time in seconds.
            if msg.time > 0:
                delta = msg.time * seconds_per_tick
            else:
                delta = 0

            yield msg.copy(time=delta)

            if msg.type == 'set_tempo':
                seconds_per_tick = self._get_seconds_per_tick(msg.tempo)

    def play(self, meta_messages=False):
        """Play back all tracks.

        The generator will sleep between each message by
        default. Messages are yielded with correct timing. The time
        attribute is set to the number of seconds slept since the
        previous message.

        By default you will only get normal MIDI messages. Pass
        meta_messages=True if you also want meta messages.

        You will receive copies of the original messages, so you can
        safely modify them without ruining the tracks.

        """
        sleep = time.sleep

        for msg in self:
            sleep(msg.time)

            if isinstance(msg, MetaMessage) and not meta_messages:
                continue
            else:
                yield msg

    def save(self, filename=None, file=None):
        """Save to a file.

        If file is passed the data will be saved to that file. This is typically
        an in-memory file or and already open file like sys.stdout.

        If filename is passed the data will be saved to that file.

        Raises ValueError if both file and filename are None,
        or if a type 0 file has != one track.
        """
        if self.type == 0 and len(self.tracks) != 1:
            raise ValueError('type 0 file must have exactly 1 track')

        if file is not None:
            self._save(file)
        elif filename is not None:
            with io.open(filename, 'wb') as file:
                self._save(file)
        else:
            raise ValueError('requires filename or file')

    def _save(self, outfile):
        with meta_charset(self.charset):
            header = struct.pack('>hhh', self.type,
                                 len(self.tracks),
                                 self.ticks_per_beat)

            write_chunk(outfile, b'MThd', header)

            for track in self.tracks:
                write_track(outfile, track)

    def print_tracks(self, meta_only=False):
        """Prints out all messages in a .midi file.

        May take argument meta_only to show only meta messages.

        Use:
        print_tracks() -> will print all messages
        print_tracks(meta_only=True) -> will print only MetaMessages
        """
        for i, track in enumerate(self.tracks):
            print('=== Track {}'.format(i))
            for msg in track:
                if not isinstance(msg, MetaMessage) and meta_only:
                    pass
                else:
                    print('{!r}'.format(msg))

    def __repr__(self):
        return '<midi file {!r} type {}, {} tracks, {} messages>'.format(
            self.filename, self.type, len(self.tracks),
            sum([len(track) for track in self.tracks]))


    # The context manager has no purpose but is kept around since it was
    # used in examples in the past.
    def __enter__(self):
        return self

    def __exit__(self, type, value, traceback):
        return False



import io
from pytest import raises
from .messages import Message
from .midifiles import MidiFile

HEADER_ONE_TRACK = """
4d 54 68 64  # MThd
00 00 00 06  # Chunk size
      00 01  # Type 1
      00 01  # 1 track
      00 78  # 120 ticks per beat
"""    


def parse_hexdump(hexdump):
    data = bytearray()
    for line in hexdump.splitlines():
        data += bytearray.fromhex(line.split('#')[0])
    return data


def read_file(hexdump):
    return MidiFile(file=io.BytesIO(parse_hexdump(hexdump)))


def test_no_tracks():
    assert read_file("""
    4d 54 68 64  # MThd
    00 00 00 06  # Chunk size
    00 01  # Type 1
    00 00  # 0 tracks
    00 78  # 120 ticks per beat
    """).tracks == []


def test_single_message():
    assert read_file(HEADER_ONE_TRACK + """
    4d 54 72 6b  # MTrk
    00 00 00 04
    20 90 40 40  # note_on
    """).tracks[0] == [Message('note_on', note=64, velocity=64, time=32)]


def test_two_tracks():
    mid = read_file("""
    4d54 6864 0000 0006 0001 0002 0040        # Header
    4d54 726b 0000 0008 00 90 40 10  40 80 40 10   # Track 0
    4d54 726b 0000 0008 00 90 47 10  40 80 47 10   # Track 1
    """)
    assert len(mid.tracks) == 2
    # Todo: add some more tests here.


def test_empty_file():
    with raises(EOFError):
        read_file("")
   

def test_eof_in_track():
    with raises(EOFError):
        read_file(HEADER_ONE_TRACK + """
        4d 54 72 6b  # MTrk
        00 00 00 01  # Chunk size
        # Oops, no data here.
        """)


def test_invalid_data_byte():
    # Todo: should this raise IOError?
    with raises(IOError):
        read_file(HEADER_ONE_TRACK + """
        4d 54 72 6b  # MTrk
        00 00 00 04  # Chunk size
        00 90 ff 40  # note_on note=255 velocity=64
        """)
